@@ -38,6 +38,7 @@ Downloader::Downloader(NzbList *nzblist, int thread_id, QMutex *file_lock, QTabl
 	this->l = l;
 	this->use_ssl = false;
         this->last_failed = false;
+        paused = false;
 	connect(this, SIGNAL(downloadEvent(QString, int, int)), parent, SLOT(downloadEvent(QString, int, int)));
 }
 
@@ -56,7 +57,8 @@ void Downloader::init()
 {
 	file = 0, seg = 0;
 	downloading = false;
-	connect(this, SIGNAL(processNext()), this, SLOT(processFiles()));
+        paused = false;
+        connect(this, SIGNAL(processNext()), this, SLOT(processFiles()));
 	
 	use_ssl = ((MainWindow*)parent)->getConfig()->value("server/ssl").toBool();
 	
@@ -88,6 +90,17 @@ void Downloader::init()
 void Downloader::processFiles()
 {
 	if (downloading) return;
+        if (paused) {
+            speedTimer.stop();
+            terminate();
+            dldata.clear();
+            //delete usenet;  Crashes Qt
+            emit downloadEvent("0", thread_id, 4);
+            emit downloadEvent("Paused", thread_id, 6);
+
+            this->exit();
+            return;
+        }
 	
 	if (getNext()) {
 			downloading = true;
@@ -148,10 +161,22 @@ void Downloader::stop()
 	emit downloadEvent("Disconnected", thread_id);
 
 	dldata.clear();
-	delete usenet;
+        //delete usenet;
 	this->exit();
 }
 
+void Downloader::pause()
+{
+    paused = true;
+}
+
+void Downloader::resume()
+{
+    // This slot is not used/supported.
+    paused = false;
+    emit processNext();
+}
+
 void Downloader::gotError(QAbstractSocket::SocketError err)
 {
 	speedTimer.stop();
@@ -183,8 +208,15 @@ void Downloader::gotConnected()
 
 void Downloader::terminate()
 {
-        usenet->write(((QString)("QUIT\r\n")).toAscii());
+        if (usenet->ConnectedState == QAbstractSocket::ConnectedState) {
+            usenet->write(((QString)("QUIT\r\n")).toAscii());
+            usenet->flush();
+        }
+
+        usenet->waitForReadyRead();
+
         usenet->close();
+        usenet->waitForDisconnected();
 	
 	((MainWindow*)parent)->getDownloaderItem(thread_id)->setIcon(QIcon(":/images/connect_no.png"));
 }
