@@ -47,6 +47,8 @@ signals:
 public slots:
 	void processFiles();
 	void stop();
+        void pause();
+        void resume();
 
 private slots:
         void init();
@@ -71,7 +73,7 @@ private:
 	int thread_id;
 	int file, seg;
 	QByteArray dldata;
-	bool downloading;
+        bool downloading, paused;
 	QTime last_time;
 	int total_bytes;
 	int last_bytes;
