@@ -34,6 +34,7 @@
 MainWindow::MainWindow()
 {
 	shutting_down = false;
+    download_complete = false;
 	
 	split = new QSplitter(Qt::Vertical, this);
 	
@@ -72,6 +73,8 @@ MainWindow::MainWindow()
 		d->setItem(i, 2, new QTableWidgetItem("0 kB/s"));
 		d->item(i, 2)->setTextAlignment(Qt::AlignRight);
 		connect(this, SIGNAL(fullStop()), downloaders[i], SLOT(stop()));
+                connect(this, SIGNAL(requestPause()), downloaders[i], SLOT(pause()));
+                connect(this, SIGNAL(requestResume()), downloaders[i], SLOT(resume()));
 	}
 
 	decoders.append(new Decoder(&nzblist, i, &file_lock, this));
@@ -98,6 +101,10 @@ void MainWindow::closeEvent(QCloseEvent *event)
 	
 	if (stopAct->isEnabled()) {
 		stop();
+        if (!shutting_down) {
+            event->ignore();
+            return;
+        }
 	}
 	
 	writeSettings();
@@ -113,7 +120,7 @@ void MainWindow::open()
 
 void MainWindow::about()
 {
-	QMessageBox::about(this, tr("About nzb"), tr("nzb v%1\n\nnzb is a Usenet binary downloader.\n\nSee http://nzb.sf.net/").arg(NZB_VERSION));
+        QMessageBox::about(this, tr("About nzb"), tr("nzb v%1\n\nnzb is a Usenet binary downloader.\n\nSee http://www.nzb.fi/").arg(NZB_VERSION));
 }
 
 void MainWindow::createActions()
@@ -156,11 +163,23 @@ void MainWindow::createActions()
 	stopAct->setEnabled(false);
 	connect(stopAct, SIGNAL(triggered()), this, SLOT(stop()));
 	
-	streamAct = new QAction(QIcon(":/images/stream.png"), tr("St&ream"), this);
+    streamAct = new QAction(QIcon(":/images/multimedia2.png"), tr("St&ream"), this);
 	streamAct->setShortcut(tr("Ctrl+R"));
 	streamAct->setStatusTip(tr("Start streaming"));
 	streamAct->setEnabled(false);
 	connect(streamAct, SIGNAL(triggered()), this, SLOT(stream()));
+
+        pauseAct = new QAction(QIcon(":/images/player_pause.png"), tr("&Pause"), this);
+        pauseAct->setShortcut(tr("Ctrl+P"));
+        pauseAct->setStatusTip(tr("Pause"));
+        pauseAct->setEnabled(false);
+        connect(pauseAct, SIGNAL(triggered()), this, SLOT(pause()));
+
+        resumeAct = new QAction(QIcon(":/images/stream.png"), tr("Resum&e"), this);
+        resumeAct->setShortcut(tr("Ctrl+E"));
+        resumeAct->setStatusTip(tr("Resume"));
+        resumeAct->setEnabled(false);
+        connect(resumeAct, SIGNAL(triggered()), this, SLOT(resume()));
 }
 
 void MainWindow::createMenus()
@@ -182,6 +201,8 @@ void MainWindow::createMenus()
 	actionsMenu->addAction(startAct);
 	actionsMenu->addAction(stopAct);
 	actionsMenu->addAction(streamAct);
+        actionsMenu->addAction(pauseAct);
+        actionsMenu->addAction(resumeAct);
 
 	menuBar()->addSeparator();
 
@@ -203,6 +224,8 @@ void MainWindow::createToolBars()
 	actionToolBar->addAction(startAct);
 	actionToolBar->addAction(stopAct);
 	actionToolBar->addAction(streamAct);
+        actionToolBar->addAction(pauseAct);
+        actionToolBar->addAction(resumeAct);
 }
 
 void MainWindow::createStatusBar()
@@ -366,9 +389,13 @@ void MainWindow::sortList()
 
 void MainWindow::start()
 {
+        download_complete = false;
+        
 	startAct->setEnabled(false);
 	stopAct->setEnabled(true);
 	streamAct->setEnabled(false);
+        pauseAct->setEnabled(true);
+        resumeAct->setEnabled(false);
 	
 	for (int i=0; i<downloaders.size(); i++) {
 		downloaders[i]->start(QThread::LowPriority);
@@ -391,9 +418,26 @@ void MainWindow::restartDownloaders()
 
 void MainWindow::stop()
 {
+    if (!download_complete) {
+        QMessageBox msgBox;
+        msgBox.setText("If you stop the current file list will be cleared. You can use the pause action to temporary stop the download.");
+        msgBox.setInformativeText("Are you sure you want to stop and clear the list?");
+        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        msgBox.setDefaultButton(QMessageBox::No);
+        msgBox.setIcon(QMessageBox::Question);
+        int ret = msgBox.exec();
+        if (ret == QMessageBox::No) {
+            shutting_down = false;
+            return;
+        }
+    }
+
 	startAct->setEnabled(true);
 	stopAct->setEnabled(false);
 	streamAct->setEnabled(true);
+        pauseAct->setEnabled(false);
+        resumeAct->setEnabled(false);
+
 	
 	qDebug("Stopping");
 	
@@ -409,7 +453,7 @@ void MainWindow::stop()
 	
 	clearData();
 
-        setWindowTitle(tr("nzb"));
+    setWindowTitle(tr("nzb"));
 }
 
 void MainWindow::clearData()
@@ -426,13 +470,30 @@ void MainWindow::clearData()
 	
 	startAct->setEnabled(false);
 	streamAct->setEnabled(false);
+        pauseAct->setEnabled(false);
+        resumeAct->setEnabled(false);
+
 }
 
 void MainWindow::stream()
 {
-	startAct->setEnabled(false);
+    if (getConfig()->value("output/mediaplayer").toString().isEmpty()) {
+        QMessageBox msgBox;
+        msgBox.setText("No media player has been specified in output settings.");
+        msgBox.setInformativeText("Please specify an application for streaming before using this function.");
+        msgBox.setStandardButtons(QMessageBox::Ok);
+        msgBox.setDefaultButton(QMessageBox::Ok);
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.exec();
+
+        return;
+    }
+
+    startAct->setEnabled(false);
 	stopAct->setEnabled(true);
 	streamAct->setEnabled(false);
+        pauseAct->setEnabled(true);
+        resumeAct->setEnabled(false);
 	
 	for (int i=0; i<downloaders.size(); i++) {
 		downloaders[i]->start(QThread::LowPriority);
@@ -444,6 +505,28 @@ void MainWindow::stream()
 	output->start(QThread::TimeCriticalPriority);
 }
 
+void MainWindow::pause()
+{
+        startAct->setEnabled(false);
+        stopAct->setEnabled(true);
+        streamAct->setEnabled(false);
+        pauseAct->setEnabled(false);
+        resumeAct->setEnabled(true);
+
+        emit requestPause();
+}
+
+void MainWindow::resume()
+{
+        startAct->setEnabled(false);
+        stopAct->setEnabled(true);
+        streamAct->setEnabled(false);
+        pauseAct->setEnabled(true);
+        resumeAct->setEnabled(false);
+
+        emit restartDownloaders();
+}
+
 void MainWindow::options()
 {
 	OptionsDlg *dl = new OptionsDlg(this);
@@ -480,8 +563,10 @@ void MainWindow::downloadEvent(QString message, int row, int type)
 		totalSpeed->setText(QString::number(total_speed)+" kB/s");
 	} else if (type == 5) {
 		d->model()->setData(d->model()->index(row, 1), message);
-		QTimer::singleShot(60000, this, SLOT(restartDownloaders()));
-	}
+                QTimer::singleShot(60000, this, SLOT(restartDownloaders()));
+        } else if (type == 6) {
+                d->model()->setData(d->model()->index(row, 1), message);
+        }
 }
 
 void MainWindow::decodeEvent(QString message, int row, int type)
@@ -501,8 +586,10 @@ void MainWindow::decodeEvent(QString message, int row, int type)
 void MainWindow::outputEvent(QString message, int type)
 {
 	if (type == 1) {
+	        download_complete = true;
 		QTimer::singleShot(200, this, SLOT(stop()));
 	} else if (type == 2) {
+	        download_complete = true;
 		QTimer::singleShot(200, this, SLOT(stop()));
 		QMessageBox *msgbox = new QMessageBox("nzb Error", message, QMessageBox::Critical, QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
 		msgbox->show();
