@@ -131,8 +131,10 @@ bool NzbHandler::startElement(const QString &, const QString &, const QString &q
 		epoch.setTime_t((attributes.value("date")).toInt());
 		NzbFile newfile(attributes.value("poster"), epoch, attributes.value("subject"));
 		currentFile = newfile;
+		fileSegs.clear();
 	} else if (qName == "segment") {
 		currentBytes = attributes.value("bytes").toInt();
+		currentSegPos = attributes.value("number").toInt();
 	}
 
 	currentText.clear();
@@ -145,9 +147,13 @@ bool NzbHandler::endElement(const QString &, const QString &, const QString &qNa
 		currentFile.addGroup(currentText);
 	} else if (qName == "segment") {
 		NzbSeg newseg(currentBytes, "<"+currentText+">");
-		currentFile.addSegment(newseg);
+		//currentFile.addSegment(newseg, currentSegPos);
+		fileSegs[currentSegPos] = newseg;
 		currentFile.setNzbFileName(nzbFileName);
         } else if (qName == "file") {
+		for (int i = 1; i <= fileSegs.size(); i++) {
+		    currentFile.addSegment(fileSegs.value(i));
+		}
 		parent->addFile(currentFile);
 	}
 	
