@@ -58,13 +58,14 @@ private slots:
 private:
 	bool getNext(int *file, int *seg);
 	void writeOut(int file, int seg);
-	bool stream;
+        bool stream, first_output;
 	QFile *fout;
 	QObject *parent;
 	NzbList *nzblist;
 	int thread_id;
 	QMutex *file_lock;
-	int file, seg;
+	int file, seg, last_file;
+	QString fn;
 	
 	QTcpServer *streamer;
 	QTcpSocket *ms;
